package javax.swing.table;

import javax.swing.UIManager;
import javax.swing.JTable;
import java.beans.PropertyChangeListener;
import java.awt.Component;
import javax.swing.event.SwingPropertyChangeSupport;
import java.io.Serializable;

public class TableColumn implements Serializable
{
    public static final String COLUMN_WIDTH_PROPERTY = "columWidth";
    public static final String HEADER_VALUE_PROPERTY = "headerValue";
    public static final String HEADER_RENDERER_PROPERTY = "headerRenderer";
    public static final String CELL_RENDERER_PROPERTY = "cellRenderer";
    protected int modelIndex;
    protected Object identifier;
    protected int width;
    protected int minWidth;
    private int preferredWidth;
    protected int maxWidth;
    protected TableCellRenderer headerRenderer;
    protected Object headerValue;
    protected TableCellRenderer cellRenderer;
    protected TableCellEditor cellEditor;
    protected boolean isResizable;
    @Deprecated
    protected transient int resizedPostingDisableCount;
    private SwingPropertyChangeSupport changeSupport;
    
    public TableColumn() {
        this(0);
    }
    
    public TableColumn(int n) {
        this(n, 75, null, null);
    }
    
    public TableColumn(int n, int n2) {
        this(n, n2, null, null);
    }
    
    public TableColumn(int modelIndex, int a, TableCellRenderer cellRenderer, TableCellEditor cellEditor) {
        this.modelIndex = modelIndex;
        int max = Math.max(a, 0);
        this.width = max;
        this.preferredWidth = max;
        this.cellRenderer = cellRenderer;
        this.cellEditor = cellEditor;
        this.minWidth = Math.min(15, this.width);
        this.maxWidth = Integer.MAX_VALUE;
        this.isResizable = true;
        this.resizedPostingDisableCount = 0;
        this.headerValue = null;
    }
    
    private void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        if (this.changeSupport != null) {
            this.changeSupport.firePropertyChange(propertyName, oldValue, newValue);
        }
    }
    
    private void firePropertyChange(String s, int i, int j) {
        if (i != j) {
            this.firePropertyChange(s, Integer.valueOf(i), Integer.valueOf(j));
        }
    }
    
    private void firePropertyChange(String s, boolean b, boolean b2) {
        if (b != b2) {
            this.firePropertyChange(s, Boolean.valueOf(b), Boolean.valueOf(b2));
        }
    }
    
    public void setModelIndex(int modelIndex) {
        this.firePropertyChange("modelIndex", this.modelIndex, this.modelIndex = modelIndex);
    }
    
    public int getModelIndex() {
        return this.modelIndex;
    }
    
    public void setIdentifier(Object identifier) {
        this.firePropertyChange("identifier", this.identifier, this.identifier = identifier);
    }
    
    public Object getIdentifier() {
        return (this.identifier != null) ? this.identifier : this.getHeaderValue();
    }
    
    public void setHeaderValue(Object headerValue) {
        this.firePropertyChange("headerValue", this.headerValue, this.headerValue = headerValue);
    }
    
    public Object getHeaderValue() {
        return this.headerValue;
    }
    
    public void setHeaderRenderer(TableCellRenderer headerRenderer) {
        this.firePropertyChange("headerRenderer", this.headerRenderer, this.headerRenderer = headerRenderer);
    }
    
    public TableCellRenderer getHeaderRenderer() {
        return this.headerRenderer;
    }
    
    public void setCellRenderer(TableCellRenderer cellRenderer) {
        this.firePropertyChange("cellRenderer", this.cellRenderer, this.cellRenderer = cellRenderer);
    }
    
    public TableCellRenderer getCellRenderer() {
        return this.cellRenderer;
    }
    
    public void setCellEditor(TableCellEditor cellEditor) {
        this.firePropertyChange("cellEditor", this.cellEditor, this.cellEditor = cellEditor);
    }
    
    public TableCellEditor getCellEditor() {
        return this.cellEditor;
    }
    
    public void setWidth(int a) {
        this.firePropertyChange("width", this.width, this.width = Math.min(Math.max(a, this.minWidth), this.maxWidth));
    }
    
    public int getWidth() {
        return this.width;
    }
    
    public void setPreferredWidth(int a) {
        this.firePropertyChange("preferredWidth", this.preferredWidth, this.preferredWidth = Math.min(Math.max(a, this.minWidth), this.maxWidth));
    }
    
    public int getPreferredWidth() {
        return this.preferredWidth;
    }
    
    public void setMinWidth(int a) {
        int minWidth = this.minWidth;
        this.minWidth = Math.max(Math.min(a, this.maxWidth), 0);
        if (this.width < this.minWidth) {
            this.setWidth(this.minWidth);
        }
        if (this.preferredWidth < this.minWidth) {
            this.setPreferredWidth(this.minWidth);
        }
        this.firePropertyChange("minWidth", minWidth, this.minWidth);
    }
    
    public int getMinWidth() {
        return this.minWidth;
    }
    
    public void setMaxWidth(int b) {
        int maxWidth = this.maxWidth;
        this.maxWidth = Math.max(this.minWidth, b);
        if (this.width > this.maxWidth) {
            this.setWidth(this.maxWidth);
        }
        if (this.preferredWidth > this.maxWidth) {
            this.setPreferredWidth(this.maxWidth);
        }
        this.firePropertyChange("maxWidth", maxWidth, this.maxWidth);
    }
    
    public int getMaxWidth() {
        return this.maxWidth;
    }
    
    public void setResizable(boolean isResizable) {
        this.firePropertyChange("isResizable", this.isResizable, this.isResizable = isResizable);
    }
    
    public boolean getResizable() {
        return this.isResizable;
    }
    
    public void sizeWidthToFit() {
        if (this.headerRenderer == null) {
            return;
        }
        Component tableCellRendererComponent = this.headerRenderer.getTableCellRendererComponent(null, this.getHeaderValue(), false, false, 0, 0);
        this.setMinWidth(tableCellRendererComponent.getMinimumSize().width);
        this.setMaxWidth(tableCellRendererComponent.getMaximumSize().width);
        this.setPreferredWidth(tableCellRendererComponent.getPreferredSize().width);
        this.setWidth(this.getPreferredWidth());
        return;
    }
    
    @Deprecated
    public void disableResizedPosting() {
        ++this.resizedPostingDisableCount;
    }
    
    @Deprecated
    public void enableResizedPosting() {
        --this.resizedPostingDisableCount;
    }
    
    public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
        if (this.changeSupport == null) {
            this.changeSupport = new SwingPropertyChangeSupport(this);
        }
        this.changeSupport.addPropertyChangeListener(listener);
    }
    
    public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
        if (this.changeSupport != null) {
            this.changeSupport.removePropertyChangeListener(listener);
        }
    }
    
    public synchronized PropertyChangeListener[] getPropertyChangeListeners() {
        if (this.changeSupport == null) {
            return new PropertyChangeListener[0];
        }
        return this.changeSupport.getPropertyChangeListeners();
    }
    
    protected TableCellRenderer createDefaultHeaderRenderer() {
        DefaultTableCellRenderer defaultTableCellRenderer = new DefaultTableCellRenderer() {
            @Override
            public Component getTableCellRendererComponent(JTable table, Object o, boolean b, boolean b2, int n, int n2) {
                if (table != null) {
                    JTableHeader tableHeader = table.getTableHeader();
                    if (tableHeader != null) {
                        this.setForeground(tableHeader.getForeground());
                        this.setBackground(tableHeader.getBackground());
                        this.setFont(tableHeader.getFont());
                    }
                }
                this.setText((o == null) ? "" : o.toString());
                this.setBorder(UIManager.getBorder("TableHeader.cellBorder"));
                return this;
            }
        };
        defaultTableCellRenderer.setHorizontalAlignment(0);
        return defaultTableCellRenderer;
    }
}
